/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * IrmpServer.java - IRMP Server class
 *
 * Copyright (c) 2014 Frank Meyer - frank(at)fli4l.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

package com.voice_tour.remotebutler;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xmlpull.v1.XmlSerializer;

import android.util.Log;
import android.util.Xml;

public class IrmpServer
{
	public static final int UDP = 0;
	public static final int TCP = 1;

	// public static int		nservers = 0;	
	public	String			name;
	public	String			ipaddress;
	public	int				ipprotocol;
	public	int				port;
	
	IrmpServer ()
	{
		// nservers++;
	}

	IrmpServer (String new_name, String new_ipaddress, int new_ipprotocol, int new_port)
	{
		this ();
		setMembers (new_name, new_ipaddress, new_ipprotocol, new_port);
	}

	private void setMembers (String na, String ip, int pr, int po)
	{
		name		= na;
		ipaddress	= ip;
		ipprotocol	= pr;
		port		= po;
	}

	public static int loadServers (IrmpServer[] servers, final int max_servers, File path, String filename)
	{
		int			rtc = 0;

	    class MyXMLHandler extends DefaultHandler
	    {
	    	private Boolean		currentElement = false;
	    	private String		currentValue = null;
			public IrmpServer[] irmpservers;
			private int			serveridx;
			private int			servers_read = 0;
			private String		tmp_name;
			private String		tmp_ipaddress;
			private int			tmp_ipprotocol;
			private int			tmp_port;
			
	    	/** Called when tag starts ( ex:- <name>AndroidPeople</name> 
	    	 * -- <name> )*/
	    	@Override
	    	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException
	    	{
	    		currentElement = true;
	
	    		if (localName.equalsIgnoreCase("server"))		// default values
	    		{
	    			tmp_name		= "";
	    			tmp_ipaddress	= "";
	    			tmp_ipprotocol	= UDP;
	    			tmp_port		= 10001;
	    		}

    			// Toast.makeText(getBaseContext(), "start localName = " + localName + " value = " + currentValue, Toast.LENGTH_SHORT).show();
	    		// if (localName.equalsIgnoreCase("showgrad"))
	    		// {
	    		//	;															// create any object here
	    		// }

	    	}
	
	    	/** Called when tag closing ( ex:- <name>AndroidPeople</name> 
	    	 * -- </name> )*/
	    	@Override
	    	public void endElement(String uri, String localName, String qName) throws SAXException
	    	{
	    		currentElement = false;
	
				// Toast.makeText(getBaseContext(), "end localName = " + localName + " value = " + currentValue, Toast.LENGTH_SHORT).show();
	    		if (localName.equalsIgnoreCase("name"))
	    		{
	    			tmp_name = currentValue;
	    		}
	    		else if (localName.equalsIgnoreCase("ipaddress"))
	    		{
	    			tmp_ipaddress = currentValue;
	    		}
	    		else if (localName.equalsIgnoreCase("protocol"))
	    		{
	    			if (currentValue.equalsIgnoreCase("TCP"))
	    			{
	    				tmp_ipprotocol = TCP;
	    			}
	    			else
	    			{
	    				tmp_ipprotocol = UDP;
	    			}
	    		}
	    		else if (localName.equalsIgnoreCase("port"))
	    		{
	    			tmp_port = Integer.parseInt(currentValue) ;
	    		}
	    		else if (localName.equalsIgnoreCase("server"))
	    		{
	    			if (serveridx < max_servers)
	    			{
	    				irmpservers[serveridx] = new IrmpServer (tmp_name, tmp_ipaddress, tmp_ipprotocol, tmp_port);
	    				servers_read++;
	    			}
	    			serveridx++;
	    		}

	    		// fm: empty currentValue!
	    		// if there is an empty value, currentValue will not change, but contains last(!) value
	    		currentValue = new String("");
	    	}
	
	    	/** Called to get tag characters ( ex:- <name>AndroidPeople</name> 
	    	 * -- to get AndroidPeople Character ) */
	    	@Override
	    	public void characters(char[] ch, int start, int length) throws SAXException
	    	{
	    		if (currentElement)
	    		{
    				currentValue = new String(ch, start, length);
	    			currentElement = false;
	    		}
	    	}
	    }
	
		try
		{
			/** Handling XML */
			SAXParserFactory spf = SAXParserFactory.newInstance();
			SAXParser sp = spf.newSAXParser();
			XMLReader xr = sp.getXMLReader();
	
			/** Create handler to handle XML Tags ( extends DefaultHandler ) */
			MyXMLHandler myXMLHandler = new MyXMLHandler();
			myXMLHandler.irmpservers = servers;
			xr.setContentHandler(myXMLHandler);
	
			// URL version:
			// URL sourceUrl = new URL("http://www.androidpeople.com/wp-content/uploads/2010/06/example.xml");
			// xr.parse(new InputSource(sourceUrl.openStream()));
			//
			// File version:
			File file = new File(path, filename);
			InputStream fIn = new FileInputStream(file);
			xr.parse(new InputSource(fIn));
			rtc = myXMLHandler.servers_read;
		}
		catch (Exception e)
		{
			// Toast.makeText(getApplicationContext(), "XML Error: " + e, Toast.LENGTH_SHORT).show();
        	Log.e("Exception", "exception in loadLocations() method");
		}

		return rtc;
	}

	public static boolean saveServers (IrmpServer[] servers, int nservers, File path, String filename)
    {
		boolean		rtc = false;
		int			idx;
        File		newxmlfile = new File(path, filename);

        try
        {
        	newxmlfile.createNewFile();
        }
        catch(IOException e)
        {
        	Log.e("IOException", "exception in saveLocations() method");
        }

        FileOutputStream fileos = null;       	

        try
        {
        	fileos = new FileOutputStream(newxmlfile);
        }
        catch(FileNotFoundException e)
        {
        	Log.e("FileNotFoundException", "cannot create FileOutputStream");
        }

        XmlSerializer serializer = Xml.newSerializer();

        try
        {
			serializer.setOutput(fileos, "UTF-8");
			serializer.startDocument(null, Boolean.valueOf(true)); 
			serializer.setFeature("http://xmlpull.org/v1/doc/features.html#indent-output", true); 

			serializer.startTag(null, "RemoteButler");
			{
				for (idx = 0; idx < nservers; idx++)
				{
					serializer.startTag(null, "server");
					{
						serializer.startTag(null, "name");
						serializer.text(servers[idx].name);
						serializer.endTag(null, "name");

						serializer.startTag(null, "ipaddress");
						serializer.text(servers[idx].ipaddress);
						serializer.endTag(null, "ipaddress");

						serializer.startTag(null, "protocol");

						if (servers[idx].ipprotocol == UDP)
						{
							serializer.text("UDP");
						}
						else
						{
							serializer.text("TCP");
						}
						
						serializer.endTag(null, "protocol");
	
						serializer.startTag(null, "port");
						serializer.text(String.valueOf(servers[idx].port));
						serializer.endTag(null, "port");
					}
					serializer.endTag(null, "server");
				}
			}
			serializer.endTag(null, "RemoteButler");
			serializer.endDocument();
			serializer.flush();
			fileos.close();
			rtc = true;
		}
        catch (Exception e)
        {
			Log.e("Exception","error occurred while creating xml file");
		}

        return rtc;
    }

};
